@@ -1,5 +1,17 @@
+############################################################
+# Magic Maze - a simple and low-tech monster-bashing maze game.
+# Copyright (C) 2004-2008 Kent Dahl
+#
+# This game is FREE as in both BEER and SPEECH. 
+# It is available and can be distributed under the terms of 
+# the GPL license (version 2) or alternatively 
+# the dual-licensing terms of Ruby itself.
+# Please see README.txt and COPYING_GPL.txt for details.
+############################################################
+
 require 'sdl'
 
+require 'magicmaze/images'
 require 'magicmaze/tile'
 
 module MagicMaze
@@ -7,6 +19,7 @@ module MagicMaze
   ################################################
   #
   class Graphics
+    include Images # Generic GFX.
     DEBUG = true
 
     GFX_PATH = 'data/gfx/'
@@ -54,9 +67,54 @@ module MagicMaze
       2 + SPRITE_WIDTH * VIEW_AREA_MAP_HEIGHT_CENTER ]
 
 
-    FADE_DURATION = 16
+    ##
+    # Singleton graphics instance.
+    def self.get_graphics(options={})
+      @graphics_instance ||= MagicMaze::Graphics.new(options)
+      @graphics_instance
+    end
+
+    def self.shutdown_graphics
+      @graphics_instance.destroy
+      @graphics_instance = nil
+    end
 
     def initialize(options={})
+      puts "Starting Magic Maze..."
+      screen_init(options)
+      early_progress
+      font_init
+
+      @progress_msg = _("Summoning") + "\n."
+      early_progress
+
+      load_background_images
+
+      @progress_msg = _("Magic Maze") + "\n."
+      early_progress
+
+      @sprite_images = load_new_sprites || load_old_sprites 
+
+      # show_message("Enter!")
+      
+      # Cached values for what is already drawn.
+      @cached_drawing = Hash.new
+      @delay_stats = Array.new
+
+      puts "Graphics initialized." if DEBUG
+    end
+
+    def destroy
+      if @delay_stats && @delay_stats.size.nonzero? then
+	puts "Delay average: " + 
+	  (@delay_stats.inject(0.0){|i,j|i+j}/@delay_stats.size).to_s
+	puts "Delay min/max: " + 
+	  @delay_stats.min.to_s + " / " + @delay_stats.max.to_s
+      end
+      SDL.quit
+    end
+
+    def screen_init(options)
       puts "Setting up graphics..." if DEBUG
       @xsize = FULLSCREEN[2]
       @ysize = FULLSCREEN[3]
@@ -64,29 +122,43 @@ module MagicMaze
       SDL.init( SDL::INIT_VIDEO )
       SDL::Mouse.hide
       SDL::WM.set_caption( _("Magic Maze"),"" )
-      SDL::WM.icon=( SDL::Surface.load("data/gfx/icon.png") )
+      # SDL::WM.icon=( SDL::Surface.load("data/gfx/icon.png") )
 
       screen_mode = SDL::HWSURFACE + SDL::DOUBLEBUF
       screen_mode += SDL::FULLSCREEN if options[:fullscreen] 
 
       @screen = SDL::setVideoMode(@xsize,@ysize, @bpp, screen_mode)
+      early_progress
 
-      font_init
-
-      show_message(_("Summoning..."))
-
-      load_background_images
-
-      show_message(_("Magic Maze"))
-
-      @sprite_images = load_new_sprites || load_old_sprites 
-
-      # show_message("Enter!")
-
-      puts "Graphics initialized." if DEBUG
+      SDL::WM.icon=( SDL::Surface.load("data/gfx/icon.png") )
+      early_progress
+      
+      unless @screen.respond_to? :draw_rect then
+	def @screen.draw_rect(x,y,w,h,c)
+	  # Workaround for older Ruby/SDL...
+	  fill_rect(x,y,   w,1, c)
+	  fill_rect(x,y,   1,h, c)
+	  fill_rect(x,y+h, w,1, c)
+	  fill_rect(x+w,y, 1,h, c)
+	end
+      end
     end
 
 
+    # Simple progress indication before we can write etc to screen.
+    def early_progress(progress=nil, flip=true, clear=true)
+      @progress = progress || (@progress||0)+1
+      w = SCALE_FACTOR * (64 - @progress*8)
+      c = 255 - (@progress**2)
+      clear_screen if clear
+      @screen.fill_rect(@xsize-w,0, w,@ysize,
+			@screen.map_rgb(c,c,c))
+      show_long_message(@progress_msg) if @progress_msg
+      @screen.flip if flip
+    end
+
+
+
     def font_init
       ## Fonts
       SDL::TTF.init
@@ -115,6 +187,7 @@ module MagicMaze
       @background_images = {}
       SCREEN_IMAGES.each{|key, filename|
         source_image = SDL::Surface.load( GFX_PATH+filename ) 
+	@progress_msg += "." ; early_progress
         if SCALE_FACTOR != 1 then
           scaled_image = SDL::Surface.new(SDL::SWSURFACE, 
                                         source_image.w * SCALE_FACTOR, 
@@ -122,7 +195,7 @@ module MagicMaze
                                           @screen)
           scaled_image.set_palette( SDL::LOGPAL|SDL::PHYSPAL, 
                                     source_image.get_palette, 0 )
-          linear_scale_image(source_image,0,0, scaled_image )
+          linear_scale_image(source_image,0,0, scaled_image, SCALE_FACTOR )
         else
           scaled_image = source_image
         end
@@ -131,8 +204,6 @@ module MagicMaze
       }
     end
 
-
-
     ##
     # reads in the old sprites from the "undocumented" format I used.
     #
@@ -201,8 +272,8 @@ module MagicMaze
 
       @screen.set_palette( SDL::LOGPAL|SDL::PHYSPAL, palette, 0 )
 
-
       (0...lines).each do|line|	
+	@progress_msg += "." ; early_progress
 	(0...10).each do|column|
 	  sprite = SDL::Surface.new(SDL::HWSURFACE, #|SDL::SRCCOLORKEY,
                                     SPRITE_WIDTH, SPRITE_HEIGHT, @screen)
@@ -219,7 +290,7 @@ module MagicMaze
           if SCALE_FACTOR == 1 then
             SDL.blitSurface(spritemap,x,y,w,h,sprite, 0,0 )
           else
-            linear_scale_image(spritemap,x,y, sprite )
+            linear_scale_image(spritemap,x,y, sprite, SCALE_FACTOR )
           end
 
 	  sprite.set_palette( mode, palette, 0 )
@@ -257,76 +328,14 @@ module MagicMaze
     end
 
 
-    ##
-    # Slow, but blocky and non-SGE.
-    #
-    def linear_scale_image( source_image, sx,sy, scaled_image, factor = SCALE_FACTOR )
-      sw = scaled_image.w / factor
-      sh = scaled_image.h / factor
-
-      source_image.lock
-      scaled_image.lock
-
-      sh.times do |sdy|        
-        sw.times do |sdx|
-          pixel = source_image.get_pixel(sx+sdx, sy+sdy)
-          scaled_image.fill_rect(sdx*factor, sdy*factor, factor, factor, pixel)
-        end
-      end
-
-      source_image.unlock
-      scaled_image.unlock
-
-      scaled_image
-    end
-
-    ######################################################
-    # general graphics methods
-
-    ## put up a background screen
-    def put_screen( screen, center = false, flip = true )
-      @screen.fillRect(0,0,@xsize,@ysize,0)
-      image = @background_images[ screen ]
-      x,y=0,0
-      if center
-        x = (@xsize - image.w)/2
-        y = (@ysize - image.h)/2        
-      end
-      @screen.put( image, x,y )
-      @screen.flip if flip
-    end
-
-    def put_background( sprite, x, y )
-      put_sprite( sprite, x, y )
-    end
-
-    def put_sprite( sprite, x, y )
-      image = @sprite_images[sprite]
-      @screen.put( image, x, y ) if image   
-    end    
-
-    def flip
-      @screen.flip
-    end
-
-    def toogle_fullscreen
-      @screen.toggle_fullscreen
-    end
-
-    def write_text( text, x, y, font = @font16 )
-      font.drawSolidUTF8(@screen,text,x,y,255,255,255)
-    end
-
-    def write_smooth_text( text, x, y, font = @font16 )
-      font.drawBlendedUTF8(@screen,text,x,y,255,255,255)
-    end
-
 
     #################################################
     # View specific methods
 
 
     def write_score( score )
+      return if cached_drawing_valid?(:score, score )
+
       text = sprintf "%9d", score   # fails on EeePC
       # text = sprintf "%09d", score # old safe one.
       rect = SCORE_RECTANGLE
@@ -389,10 +398,18 @@ module MagicMaze
       @screen.flip if flip
     end
 
+    def cached_drawing_valid?(symbol, value)
+      return true if value == @cached_drawing[symbol]
+      @cached_drawing[symbol] = value
+      false
+    end
+
     
     ##
     # assumes life and mana are in range (0..100)
     def update_life_and_mana( life, mana )
+      return if cached_drawing_valid?(:life_and_mana, life.hash ^ mana.hash )
+
       rect = LIFE_MANA_RECTANGLE
       @screen.fillRect(*rect) 
       @screen.fillRect(rect[0], rect[1], 
@@ -404,6 +421,8 @@ module MagicMaze
     end
 
     def update_inventory( inventory )
+      # FIXME: return if cached_drawing_valid?(:inventory, inventory.hash )
+
       rect = INVENTORY_RECTANGLE
       @screen.fillRect(*rect) 
       currx = rect.first
@@ -416,6 +435,8 @@ module MagicMaze
     end
 
     def update_spells( primary, secondary )
+      return if cached_drawing_valid?(:spells, primary.hash ^ secondary.hash )
+
       rect1 = SPELL_RECTANGLE
       rect2 = ALT_SPELL_RECTANGLE
       @screen.fillRect( *rect1 )
@@ -457,56 +478,6 @@ module MagicMaze
     end
 
 
-    ####################################
-    #
-
-    def set_palette( pal, start_color = 0 )
-      pal ||= @sprite_palette
-      @screen.set_palette( SDL::PHYSPAL, pal, start_color )
-    end
-
-    def fade_out( tr = 0, tg = 0, tb = 0 )
-      mypal = @sprite_palette.dup
-      @old_palette = mypal
-      range = FADE_DURATION
-      (0...range).each {|i|
-	factor = (range-i).to_f / range
-	set_palette( mypal.map {|r,g,b| 
-		      [ ( r - tr ) * factor + tr,
-			( g - tg ) * factor + tg, 
-			( b - tb ) * factor + tb ]
-		    } )
-	yield i, range
-      }
-      @fade_color = [ tr, tg, tb ]
-    end
-
-    def fade_in
-      mypal = @old_palette || @sprite_palette
-      tr, tg, tb = *(@fade_color || [0,0,0])
-      range = FADE_DURATION
-      (0..range).each {|i|
-	factor = i.to_f / range
-	set_palette( mypal.map {|r,g,b| 
-		      [ ( r - tr ) * factor + tr,
-			( g - tg ) * factor + tg, 
-			( b - tb ) * factor + tb ]
-		    } )
-	yield i, range
-      }
-      set_palette( mypal )
-    end
-
-    def fade_in_and_out( sleep_ms = 500, &block )
-      fade_in( &block )
-      SDL.delay( sleep_ms )
-      fade_out( &block )      
-    end
-
-    def clear_screen
-      @screen.fillRect( 0, 0, @xsize, @ysize, 0 )
-    end
-
     ####################################
     #
     def show_help
@@ -519,7 +490,7 @@ module MagicMaze
 	'Alt :-  Cast secondary spell',
 	'X / Z :- Toggle attack spell',
 	'A / S :- Toggle secondary spell',
-	'',
+	'', # Failed for RubySDL2.0.1 and Ruby1.9.1-p1
 	'Esc / Q :- Quit playing',
 	'F9 / R :- Restart level',
 	# '[F4]: Load game    [F5]: Save game',
@@ -531,7 +502,7 @@ module MagicMaze
       y_offset = 0
       font = @font16
       lines.each{|line|
-	write_smooth_text( line, 5, y_offset, font )
+	write_smooth_text( line, 5, y_offset, font ) if line.size.nonzero? # Failed for RubySDL2.0.1 and Ruby1.9.1-p1 on empty string.
 	y_offset+= font.height
       }
       
@@ -541,14 +512,16 @@ module MagicMaze
 
     ####################################
     #
-    def draw_map( player )
+    def draw_map( player, line_by_line = true )
       map = player.location.map
 
       rect = MAZE_VIEW_RECTANGLE
       @screen.fillRect(*rect)
       
-      @screen.flip
-      @screen.fillRect(*rect)
+      if line_by_line then
+	@screen.flip 
+	@screen.fillRect(*rect)
+      end
 
       map_zoom_factor = 4
 
@@ -556,13 +529,6 @@ module MagicMaze
       map_height = VIEW_AREA_MAP_HEIGHT * map_zoom_factor
       map_width  = VIEW_AREA_MAP_WIDTH  * map_zoom_factor
 
-      # The player
-      @screen.fill_rect(rect[0] + map_width/2  * map_block_size,
-                        rect[1] + map_height/2 * map_block_size,
-                        map_block_size,
-                        map_block_size,
-                        COL_BLUE)
-
       (0...map_height).each do |ay|
         my = ay + player.location.y - map_height/2
         draw_y = rect[1] + ay*map_block_size
@@ -576,7 +542,8 @@ module MagicMaze
             col = nil
             col = COL_LIGHTGRAY   if background.blocked? 
             col = COL_YELLOW      if entity.kind_of?( DoorTile )
-            col = COL_RED         if entity.kind_of?( Monster )            
+            col = COL_RED         if entity.kind_of?( Monster )
+            col = COL_BLUE        if entity.kind_of?( Player )
           end
           if col then
             @screen.fill_rect(rect[0] + ax*map_block_size,
@@ -587,7 +554,15 @@ module MagicMaze
           end	
 
         end
-        flip
+
+	# The center.
+	@screen.draw_rect(rect[0] + map_width/2  * map_block_size,
+			  rect[1] + map_height/2 * map_block_size,
+			  map_block_size,
+			  map_block_size,
+			  COL_WHITE)
+
+        flip if line_by_line
 
       end
 
@@ -668,15 +643,130 @@ module MagicMaze
       @screen.set_palette( SDL::PHYSPAL|SDL::LOGPAL, pal, @rotating_palette_range.first )
     end
 
+    ##
+    # Prepare menu for rendering.
+    #
     def setup_menu( entries, chosen = nil)
       @menu_items = entries
+
+      max_width = 0
+      total_height = 0
+      font = @font32
+      @menu_items.each do |text|
+	tw, th = font.text_size( text )
+	max_width = [max_width,tw+16*SCALE_FACTOR].max
+	total_height += th + 4*SCALE_FACTOR
+      end
+      @menu_width = max_width
+      @menu_chosen_item = chosen || @menu_items.first
+      
+      # Truncate if the items can fit on screen.
+      scr_height = 200 * SCALE_FACTOR
+      if total_height > scr_height then
+	@menu_height = scr_height
+	@menu_truncate_size = (@menu_items.size * scr_height / (total_height)).to_i
+      else
+	@menu_height = total_height
+	@menu_truncate_size = false 
+      end
     end
 
-    def draw_menu
-      @menu.items.each do
+    ##
+    # This does a generic menu event loop
+    #
+    def choose_from_menu( menu_items = %w{OK Cancel}, input = nil )
+      setup_menu(menu_items)
+      begin
+	draw_menu
+	menu_event = input ? input.get_menu_item_navigation_event : yield
+	if [:previous_menu_item, :next_menu_item].include?(menu_event) then
+	  self.send(menu_event)
+	end
+      end until [:exit_menu, :select_menu_item].include?(menu_event)
+      erase_menu
+      if menu_event == :select_menu_item then
+	return menu_chosen_item
+      else
+	return false
       end
     end
 
+
+    ##
+    # Draw an updated menu.
+    def draw_menu
+      topx = 160 * SCALE_FACTOR - @menu_width  / (2)
+      topy = 120 * SCALE_FACTOR - @menu_height / (2)
+
+      #TODO: Save the old background.
+
+      # Handle the case of truncated menu. Not too nice.
+      if @menu_truncate_size then
+	chosen_index = @menu_items.index(@menu_chosen_item)
+	if chosen_index then
+	  half_trunc = @menu_truncate_size / 2
+	  first_item = [chosen_index-half_trunc, 0].max
+	  if first_item.zero?
+	    half_trunc += half_trunc - chosen_index
+	  end
+	  last_item  = [chosen_index+half_trunc, @menu_items.size].min
+
+	  curr_menu_items = @menu_items[first_item..last_item]
+	else
+	  curr_menu_items = @menu_items[0..@menu_truncate_size]
+	end
+      else
+	curr_menu_items = @menu_items
+      end
+
+      @screen.fillRect( topx, topy, @menu_width,@menu_height,0 )
+      @screen.draw_rect( topx, topy, @menu_width,@menu_height, COL_GRAY )
+      y_offset = topy
+      font = @font32
+      curr_menu_items.each do |text|
+	tw, th = font.text_size( text )
+	color_intensity = 127
+	if text == @menu_chosen_item then
+	  rect = [ 
+	    topx + 4*SCALE_FACTOR, 
+	    y_offset + 4*SCALE_FACTOR,
+	    @menu_width - 8*SCALE_FACTOR, 
+	    font.height - 4*SCALE_FACTOR,
+	    COL_WHITE
+	  ]
+	  @screen.draw_rect( *rect )
+	  color_intensity = 255
+	end
+	write_smooth_text(text, 
+			  topx + (@menu_width-tw)/2, 
+			  y_offset + 2*SCALE_FACTOR, 
+			  font, *[color_intensity]*3 )
+	y_offset+= font.height + 4*SCALE_FACTOR
+      end
+      flip
+    end
+
+    attr_reader :menu_chosen_item
+
+    def previous_menu_item
+      @menu_chosen_item = 
+	@menu_items[@menu_items.index(@menu_chosen_item)-1] ||
+	@menu_items.last
+    end
+
+    def next_menu_item
+      @menu_chosen_item = 
+	@menu_items[@menu_items.index(@menu_chosen_item)+1] ||
+	@menu_items.first
+    end
+
+
+    ##
+    # Erase the menu.
+    def erase_menu
+      # TODO: Restore background
+    end
+
   end # Graphics
 
 end
